from typing import Any, Dict, Optional, List, Union
from sklearn import preprocessing, pipeline, compose
from sklearn.preprocessing import LabelEncoder
from sklearn.base import BaseEstimator

import numpy as np
from onnxruntime import InferenceSession
from skl2onnx import convert_sklearn
from skl2onnx.common.data_types import (
    FloatTensorType,
    DoubleTensorType,
    Int64TensorType,
    Int32TensorType,
    Int8TensorType,
    UInt8TensorType,
    BooleanTensorType,
    StringTensorType,
)
import jaqpotpy
from jaqpotpy.datasets.jaqpotpy_dataset import JaqpotpyDataset
from jaqpotpy.descriptors.base_classes import MolecularFeaturizer
from jaqpotpy.api.get_installed_libraries import get_installed_libraries
from jaqpotpy.api.openapi.models import (
    FeatureType,
    FeaturePossibleValue,
    ModelType,
    ModelExtraConfig,
    Transformer,
    ModelTask,
)
from jaqpotpy.models.base_classes import Model
from jaqpotpy.doa.doa import DOA


class SklearnModel(Model):
    def __init__(
        self,
        dataset: JaqpotpyDataset,
        model: Any,
        doa: Optional[DOA or list] = None,
        preprocess_x: Optional[Union[BaseEstimator, List[BaseEstimator]]] = None,
        preprocess_y: Optional[Union[BaseEstimator, List[BaseEstimator]]] = None,
    ):
        self.dataset = dataset
        self.featurizer = dataset.featurizer
        self.model = model
        self.pipeline = None
        self.trained_model = None
        self.doa = doa if isinstance(doa, list) else [doa] if doa else []
        self.preprocess_x = (
            preprocess_x if isinstance(preprocess_x, list) else [preprocess_x]
        )
        SklearnModel.check_preprocessor(self.preprocess_x, feat_type="X")
        self.preprocess_y = (
            preprocess_y if isinstance(preprocess_y, list) else [preprocess_y]
        )
        SklearnModel.check_preprocessor(self.preprocess_y, feat_type="y")
        self.transformers_y = {}
        self.libraries = None
        self.jaqpotpy_version = jaqpotpy.__version__
        self.task = self.dataset.task
        self.initial_types = None
        self.onnx_model = None
        self.onnx_opset = None
        self.type = ModelType("SKLEARN")
        self.independentFeatures = None
        self.dependentFeatures = None
        self.extra_config = ModelExtraConfig()

    def _dtypes_to_jaqpotypes(self):
        for feature in self.independentFeatures + self.dependentFeatures:
            if feature["featureType"] in ["SMILES"]:
                feature["featureType"] = FeatureType.SMILES
            elif feature["featureType"] in ["int", "int64"]:
                feature["featureType"] = FeatureType.INTEGER
            elif feature["featureType"] in ["float", "float64"]:
                feature["featureType"] = FeatureType.FLOAT
            elif feature["featureType"] in ["string, object", "O"]:
                feature["featureType"] = FeatureType.CATEGORICAL
                categories = self.dataset.df[feature["key"]].unique()
                feature["possible_values"] = list(
                    FeaturePossibleValue(key=category, value=category)
                    for category in categories
                )

    def _extract_attributes(self, trained_class, trained_class_type):
        if trained_class_type == "doa":
            attributes = trained_class._doa_attributes
        elif trained_class_type == "featurizer":
            attributes = self.dataset.featurizers_attributes.get(
                trained_class.__class__.__name__
            )
        else:
            attributes = trained_class.__dict__
        return {
            k: (
                v.tolist()
                if isinstance(v, np.ndarray)
                else v.item()
                if isinstance(v, (np.int64, np.float64))
                else v
            )
            for k, v in attributes.items()
        }

    def _add_class_to_extraconfig(self, added_class, added_class_type):
        configurations = {}

        for attr_name, attr_value in self._extract_attributes(
            added_class, added_class_type
        ).items():
            configurations[attr_name] = attr_value

        if added_class_type == "preprocessor":
            self.extra_config.preprocessors.append(
                Transformer(name=added_class.__class__.__name__, config=configurations)
            )
        elif added_class_type == "featurizer":
            self.extra_config.featurizers.append(
                Transformer(name=added_class.__class__.__name__, config=configurations)
            )
        elif added_class_type == "doa":
            self.extra_config.doa.append(
                Transformer(name=added_class.__class__.__name__, config=configurations)
            )

    def _map_onnx_dtype(self, dtype, shape=1):
        if dtype == "int64":
            return Int64TensorType(shape=[None, shape])
        elif dtype == "int32":
            return Int32TensorType(shape=[None, shape])
        elif dtype == "int8":
            return Int8TensorType(shape=[None, shape])
        elif dtype == "uint8":
            return UInt8TensorType(shape=[None, shape])
        elif dtype == "bool":
            return BooleanTensorType(shape=[None, shape])
        elif dtype == "float32":
            return FloatTensorType(shape=[None, shape])
        elif dtype == "float64":
            return DoubleTensorType(shape=[None, shape])
        elif dtype in ["string", "object", "category"]:
            return StringTensorType(shape=[None, shape])
        else:
            return None

    def _create_onnx(self, onnx_options: Optional[Dict] = None):
        name = self.model.__class__.__name__ + "_ONNX"
        self.initial_types = []
        dtype_array = self.dataset.X.dtypes.values
        dtype_str_array = np.array([str(dtype) for dtype in dtype_array])
        all_numerical = all(
            dtype
            in [
                "int8",
                "int16",
                "int32",
                "int64",
                "uint8",
                "uint16",
                "uint32",
                "uint64",
                "float16",
                "float32",
                "float64",
                "bool",
            ]
            for dtype in dtype_str_array
        )
        if all_numerical:
            self.initial_types = [
                (
                    "input",
                    self._map_onnx_dtype("float32", len(self.dataset.X.columns)),
                )
            ]
        else:
            for i, feature in enumerate(self.dataset.X.columns):
                self.initial_types.append(
                    (
                        self.dataset.X.columns[i],
                        self._map_onnx_dtype(self.dataset.X[feature].dtype.name),
                    )
                )

        self.onnx_model = convert_sklearn(
            self.trained_model,
            initial_types=self.initial_types,
            name=name,
            options=onnx_options,
        )
        self.onnx_opset = self.onnx_model.opset_import[0].version

    def fit(self, onnx_options: Optional[Dict] = None):
        self.libraries = get_installed_libraries()
        if isinstance(self.featurizer, (MolecularFeaturizer, list)):
            if not isinstance(self.featurizer, list):
                self.featurizer = [self.featurizer]
            self.extra_config.featurizers = []
            for featurizer_i in self.featurizer:
                self._add_class_to_extraconfig(featurizer_i, "featurizer")

        if self.dataset.y is None:
            raise TypeError(
                "dataset.y is None. Please provide a target variable for the model."
            )
        # Get X and y from dataset
        X = self.dataset.__get_X__()
        y = self.dataset.__get_Y__()
        if len(self.dataset.y_cols) == 1:
            y = y.to_numpy().ravel()

        if self.doa:
            self.extra_config.doa = []
            # if not isinstance(self.doa, list):
            #     self.doa = [self.doa]
            for doa_method in self.doa:
                doa_method.fit(X=X)
                self._add_class_to_extraconfig(doa_method, "doa")

        #  Build preprocessing pipeline that ends up with the model
        self.pipeline = pipeline.Pipeline(steps=[])
        if self.preprocess_x[0] is not None:
            for preprocessor in self.preprocess_x:
                self.pipeline.steps.append((str(preprocessor), preprocessor))
        self.pipeline.steps.append(("model", self.model))

        # Apply preprocessing of response vector y
        if self.preprocess_y[0] is not None:
            if (
                self.task == "BINARY_CLASSIFICATION"
                or self.task == "MULTICLASS_CLASSIFICATION"
            ) and not isinstance(self.preprocess_y[0], LabelEncoder):
                raise ValueError(
                    "Target labels cannot be preprocessed for classification tasks. Remove any assigned preprocessing for y."
                )
            else:
                self.extra_config.preprocessors = []
                y = self.dataset.__get_Y__()
                for preprocessor in self.preprocess_y:
                    y = preprocessor.fit_transform(y)
                    self._add_class_to_extraconfig(preprocessor, "preprocessor")
                if len(self.dataset.y_cols) == 1:
                    y = y.ravel()
        self.trained_model = self.pipeline.fit(X, y)

        if self.dataset.smiles_cols:
            self.independentFeatures = list(
                {
                    "key": self.dataset.smiles_cols[smiles_col_i],
                    "name": self.dataset.smiles_cols[smiles_col_i],
                    "featureType": "SMILES",
                }
                for smiles_col_i in range(len(self.dataset.smiles_cols))
            )
        else:
            self.independentFeatures = list()
        if self.dataset.x_cols:
            self.independentFeatures += list(
                {"key": feature, "name": feature, "featureType": X[feature].dtype}
                for feature in self.dataset.active_features
            )
        self.dependentFeatures = list(
            {
                "key": feature,
                "name": feature,
                "featureType": self.dataset.__get_Y__()[feature].dtype,
            }
            for feature in self.dataset.y_cols
        )
        self._dtypes_to_jaqpotypes()

        self._create_onnx(onnx_options=onnx_options)

    def predict(self, dataset: JaqpotpyDataset):
        if not isinstance(dataset, JaqpotpyDataset):
            raise TypeError("Expected dataset to be of type JaqpotpyDataset")
        sklearn_prediction = self.trained_model.predict(
            dataset.X[self.dataset.active_features]
        )
        if self.preprocess_y[0] is not None:
            for func in self.preprocess_y[::-1]:
                if len(self.dataset.y_cols) == 1:
                    sklearn_prediction = func.inverse_transform(
                        sklearn_prediction.reshape(-1, 1)
                    ).flatten()
                else:
                    sklearn_prediction = func.inverse_transform(sklearn_prediction)
        return sklearn_prediction

    def predict_proba(self, dataset: JaqpotpyDataset):
        if not isinstance(dataset, JaqpotpyDataset):
            raise TypeError("Expected dataset to be of type JaqpotpyDataset")
        if self.task == "regression":
            raise ValueError("predict_proba is available only for classification tasks")

        sklearn_probs = self.trained_model.predict_proba(
            dataset.X[self.dataset.active_features]
        )

        sklearn_probs_list = [
            max(sklearn_probs[instance]) for instance in range(len(sklearn_probs))
        ]
        return sklearn_probs_list

    def predict_onnx(self, dataset: JaqpotpyDataset):
        if not isinstance(dataset, JaqpotpyDataset):
            raise TypeError("Expected dataset to be of type JaqpotpyDataset")
        sess = InferenceSession(self.onnx_model.SerializeToString())
        if len(self.initial_types) == 1:
            input_dtype = (
                "float32"
                if isinstance(self.initial_types[0][1], FloatTensorType)
                else "string"
            )
            input_data = {
                sess.get_inputs()[0].name: dataset.X.values.astype(input_dtype)
            }
        else:
            input_data = {
                sess.get_inputs()[i].name: dataset.X[
                    self.initial_types[i][0]
                ].values.reshape(-1, 1)
                for i in range(len(self.initial_types))
            }
        onnx_prediction = sess.run(None, input_data)
        if len(self.dataset.y_cols) == 1:
            onnx_prediction[0] = onnx_prediction[0].reshape(-1, 1)
        if self.preprocess_y[0] is not None:
            for func in self.preprocess_y[::-1]:
                onnx_prediction[0] = func.inverse_transform(onnx_prediction[0])
        if len(self.dataset.y_cols) == 1:
            return onnx_prediction[0].flatten()
        return onnx_prediction[0]

    def predict_proba_onnx(self, dataset: JaqpotpyDataset):
        if not isinstance(dataset, JaqpotpyDataset):
            raise TypeError("Expected dataset to be of type JaqpotpyDataset")
        if self.task == "regression":
            raise ValueError(
                "predict_onnx_proba is available only for classification tasks"
            )
        sess = InferenceSession(self.onnx_model.SerializeToString())
        if len(self.initial_types) == 1:
            input_dtype = (
                "float32"
                if isinstance(self.initial_types[0][1], FloatTensorType)
                else "string"
            )
            input_data = {
                sess.get_inputs()[0].name: dataset.X.values.astype(input_dtype)
            }
        else:
            input_data = {
                sess.get_inputs()[i].name: dataset.X[
                    self.initial_types[i][0]
                ].values.reshape(-1, 1)
                for i in range(len(self.initial_types))
            }
        onnx_probs = sess.run(None, input_data)
        onnx_probs_list = [
            max(onnx_probs[1][instance].values())
            for instance in range(len(onnx_probs[1]))
        ]
        return onnx_probs_list

    def deploy_on_jaqpot(self, jaqpot, name, description, visibility):
        jaqpot.deploy_sklearn_model(
            model=self, name=name, description=description, visibility=visibility
        )

    @staticmethod
    def check_preprocessor(preprocessor_list: List, feat_type: str):
        # Get all valid preprocessing classes from sklearn.preprocessing
        valid_preprocessing_classes_1 = [
            getattr(preprocessing, name)
            for name in dir(preprocessing)
            if isinstance(getattr(preprocessing, name), type)
        ]
        valid_preprocessing_classes_2 = [
            getattr(compose, name)
            for name in dir(compose)
            if isinstance(getattr(compose, name), type)
        ]
        valid_preprocessing_classes = (
            valid_preprocessing_classes_1 + valid_preprocessing_classes_2
        )

        for preprocessor in preprocessor_list:
            # Check if preprocessor is an instance of one of these classes
            if (
                not isinstance(preprocessor, tuple(valid_preprocessing_classes))
                and preprocessor is not None
            ):
                if feat_type == "X":
                    raise ValueError(
                        f"Feature preprocessing must be an instance of a valid class from sklearn.preprocessing, but got {type(preprocessor)}."
                    )
                elif feat_type == "y":
                    raise ValueError(
                        f"Response preprocessing must be an instance of a valid class from sklearn.preprocessing, but got {type(preprocessor)}."
                    )



from abc import ABC, abstractmethod, ABCMeta
import torch
import logging
import sys

import requests
import os
from typing import List, Optional, Union

# from jaqpotpy.schemas import Feature, Library, Organization
from jaqpotpy.utils.installed_packages import get_installed_packages
import inspect
from torch.optim.lr_scheduler import LambdaLR


class TorchModelTrainerMeta(ABCMeta):
    def __new__(cls, name, bases, dct):
        """
        A metaclass for torch model training, ensuring:
        - 'get_model_type' method is defined as a class method
        """
        if "get_model_type" in dct:
            method = dct["get_model_type"]
            if not isinstance(method, classmethod):
                raise TypeError(f"{name}.get_model_type must be a class method")
        return super().__new__(cls, name, bases, dct)


class TorchModelTrainer(ABC, metaclass=TorchModelTrainerMeta):
    """
    An abstract class for training a model and deploying it on Jaqpot.

    Attributes:
        model (torch.nn.Module): The torch model to be trained.
        n_epochs (int): Number of training epochs.
        optimizer (torch.optim.Optimizer): The optimizer used for training the model.
        loss_fn (torch.nn.Module): The loss function used for training.
        scheduler (torch.optim.lr_scheduler.LRScheduler): The scheduler used for adjusting the learning rate during training.
        device (torch.device): The device on which to train the model.
        use_tqdm (bool): Whether to use tqdm for progress bars.
        current_epoch (int): The epoch on which the trainer has currently reached.
        log_enabled (bool): Whether logging is enabled.
        log_filepath (os.path.relpath or None): Relative path to the log file.
        json_data_for_deployment (dict or None): The data to be sent to the API of Jaqpot in JSON format. Note that `prepare_for_deployment` must be called to compute this attribute.
        logger (logging.Logger): The logger object at INFO level used for logging during model training.
    """

    @classmethod
    @abstractmethod
    def get_model_type(cls):
        """
        Return the type of the model as a string.

        Returns:
            str: The model type.
        """
        pass

    def __init__(
        self,
        model: torch.nn.Module,
        n_epochs: int,
        optimizer: torch.optim.Optimizer,
        loss_fn: torch.nn.Module,
        scheduler: Optional[torch.optim.lr_scheduler.LRScheduler] = None,
        device: str = "cpu",
        use_tqdm: bool = True,
        log_enabled: bool = True,
        log_filepath: Optional[str] = None,
    ):
        """
        Args:
            model (torch.nn.Module): The torch model to be trained.
            n_epochs (int): Number of training epochs.
            optimizer (torch.optim.Optimizer): The optimizer used for training the model.
            loss_fn (torch.nn.Module): The loss function used for training.
            scheduler (torch.optim.lr_scheduler.LRScheduler): The scheduler used for adjusting the learning rate during training. Default is None.
            device (str, optional): The device on which to train the model. Default is 'cpu'.
            use_tqdm (bool, optional): Whether to use tqdm for progress bars. Default is True.
            log_enabled (bool, optional): Whether logging is enabled. Default is True.
            log_filepath (str or None, optional): Path to the log file. If None, logging is not saved to a file. Default is None.
        """
        self.model = model
        self.n_epochs = n_epochs
        self.optimizer = optimizer
        self.scheduler = (
            scheduler
            if scheduler is not None
            else LambdaLR(optimizer, lr_lambda=lambda epoch: 1)
        )
        self.loss_fn = loss_fn
        self.device = torch.device(device)
        self.use_tqdm = use_tqdm
        self.current_epoch = 0
        self.log_enabled = log_enabled
        self.log_filepath = (
            os.path.relpath(log_filepath) if log_filepath else log_filepath
        )
        self.json_data_for_deployment = None

        self.logger = self._setup_logger()

        self.model.to(self.device)

    def _setup_logger(self):
        """
        Setup the logger for the training process.
        """

        logger = logging.getLogger(__name__)
        logger.setLevel(logging.INFO)

        # Remove existing handlers
        for handler in logger.handlers[:]:
            logger.removeHandler(handler)

        handlers = [logging.StreamHandler(sys.stdout)]
        if self.log_filepath:
            if os.path.exists(self.log_filepath):
                raise ValueError(f"File already exists: {self.log_filepath}")
            log_file_handler = logging.FileHandler(self.log_filepath)
            handlers.append(log_file_handler)

        log_format = "%(message)s"
        formatter = logging.Formatter(log_format)
        for handler in handlers:
            handler.setFormatter(formatter)
            logger.addHandler(handler)

        return logger

    @classmethod
    def collect_subclass_info(cls):
        """
        Collect information about all subclasses of the current class.

        Returns:
            dict: A dictionary containing information about the subclasses (model type, parent class, if is abstract class or not).
        """
        subclass_info = {}

        for subclass in cls.__subclasses__():

            subclass_info[subclass.__name__] = {
                "model_type": subclass.get_model_type(),
                "parent": cls.__name__,
                "is_abstract": inspect.isabstract(subclass),
            }

            subclass_info.update(subclass.collect_subclass_info())

        return subclass_info

    @classmethod
    def get_subclass_model_types(cls):
        """
        Return the list of all the types of models that inherit from TorchModelTrainer and are implemented in the jaqpotpy_torch.trainers submodule.

        Returns:
            list: A list of model types as strings.
        """
        return [
            v["model_type"]
            for v in cls.collect_subclass_info().values()
            if not v["is_abstract"] and v["model_type"] is not None
        ]

    # @check_reach_epoch_limit
    @abstractmethod
    def train(self, train_loader, val_loader=None):
        """
        Train the model.

        Args:
            train_loader (Union[torch.utils.data.DataLoader, torch_geometric.loader.DataLoader]): DataLoader for the training dataset.
            val_loader (Union[torch.utils.data.DataLoader, torch_geometric.loader.DataLoader], optional): DataLoader for the validation dataset.
        Returns:
            None
        """
        pass

    @abstractmethod
    def evaluate(self, val_loader):
        """
        Evaluate the model.

        Args:
            loader (Union[torch.utils.data.DataLoader, torch_geometric.loader.DataLoader]): DataLoader for the evaluation dataset.
        """
        pass



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.models.error_code import ErrorCode

class TestErrorCode(unittest.TestCase):
    """ErrorCode unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testErrorCode(self):
        """Test ErrorCode"""
        # inst = ErrorCode()

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.api.dataset_api import DatasetApi


class TestDatasetApi(unittest.TestCase):
    """DatasetApi unit test stubs"""

    def setUp(self) -> None:
        self.api = DatasetApi()

    def tearDown(self) -> None:
        pass

    def test_get_dataset_by_id(self) -> None:
        """Test case for get_dataset_by_id

        Get a Dataset
        """
        pass

    def test_get_datasets(self) -> None:
        """Test case for get_datasets

        Get Datasets by User ID
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.api.model_api import ModelApi


class TestModelApi(unittest.TestCase):
    """ModelApi unit test stubs"""

    def setUp(self) -> None:
        self.api = ModelApi()

    def tearDown(self) -> None:
        pass

    def test_create_model(self) -> None:
        """Test case for create_model

        Create a new model
        """
        pass

    def test_delete_model_by_id(self) -> None:
        """Test case for delete_model_by_id

        Delete a Model
        """
        pass

    def test_get_legacy_model_by_id(self) -> None:
        """Test case for get_legacy_model_by_id

        Get a legacy model
        """
        pass

    def test_get_model_by_id(self) -> None:
        """Test case for get_model_by_id

        Get a Model
        """
        pass

    def test_get_models(self) -> None:
        """Test case for get_models

        Get paginated models
        """
        pass

    def test_get_shared_models(self) -> None:
        """Test case for get_shared_models

        Get paginated shared models
        """
        pass

    def test_partially_update_model(self) -> None:
        """Test case for partially_update_model

        Partially update specific fields of a model
        """
        pass

    def test_predict_with_model(self) -> None:
        """Test case for predict_with_model

        Predict with Model
        """
        pass

    def test_predict_with_model_csv(self) -> None:
        """Test case for predict_with_model_csv

        Predict using CSV with Model
        """
        pass

    def test_search_models(self) -> None:
        """Test case for search_models

        Search for models
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.models.partially_update_model_feature_request import PartiallyUpdateModelFeatureRequest

class TestPartiallyUpdateModelFeatureRequest(unittest.TestCase):
    """PartiallyUpdateModelFeatureRequest unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PartiallyUpdateModelFeatureRequest:
        """Test PartiallyUpdateModelFeatureRequest
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PartiallyUpdateModelFeatureRequest`
        """
        model = PartiallyUpdateModelFeatureRequest()
        if include_optional:
            return PartiallyUpdateModelFeatureRequest(
                name = 'Updated Feature Name',
                units = 'mg/L',
                description = 'An updated description for this feature',
                feature_type = 'FLOAT',
                possible_values = [
                    jaqpotpy.api.openapi.models.feature_possible_value.FeaturePossibleValue(
                        key = 'value', 
                        value = 'value', )
                    ]
            )
        else:
            return PartiallyUpdateModelFeatureRequest(
                name = 'Updated Feature Name',
                feature_type = 'FLOAT',
        )
        """

    def testPartiallyUpdateModelFeatureRequest(self):
        """Test PartiallyUpdateModelFeatureRequest"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.models.organization import Organization

class TestOrganization(unittest.TestCase):
    """Organization unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> Organization:
        """Test Organization
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `Organization`
        """
        model = Organization()
        if include_optional:
            return Organization(
                id = 56,
                name = 'my-awesome-org',
                creator = jaqpotpy.api.openapi.models.user.User(
                    id = '', 
                    username = '', 
                    first_name = '', 
                    last_name = '', 
                    email = '', 
                    email_verified = True, ),
                visibility = 'PUBLIC',
                description = 'An awesome organization for managing models.',
                organization_members = [
                    jaqpotpy.api.openapi.models.organization_user.OrganizationUser(
                        id = 56, 
                        user_id = '', 
                        username = '', 
                        email = '', 
                        association_type = 'ADMIN', )
                    ],
                contact_email = 'contact@my-awesome-org.com',
                contact_phone = '+1234567890',
                website = 'http://www.my-awesome-org.com',
                address = '123 Organization St., City, Country',
                can_edit = True,
                is_member = True,
                created_at = '',
                updated_at = ''
            )
        else:
            return Organization(
                name = 'my-awesome-org',
                visibility = 'PUBLIC',
                contact_email = 'contact@my-awesome-org.com',
        )
        """

    def testOrganization(self):
        """Test Organization"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



import hashlib
import os
import struct
from typing import Any, List, Optional, Tuple, Union
from jaqpotpy.cfg import config

import numpy as np


def np_random(
    seed: Optional[int] = config.global_seed,
) -> Tuple["RandomNumberGenerator", Any]:
    if seed is not None and not (isinstance(seed, int) and 0 <= seed):
        raise Exception(f"Seed must be a non-negative integer or omitted, not {seed}")

    seed_seq = np.random.SeedSequence(seed)
    np_seed = seed_seq.entropy
    rng = RandomNumberGenerator(np.random.PCG64(seed_seq))
    return rng, np_seed


class RandomNumberGenerator(np.random.Generator):
    def rand(self, *size):
        return self.random(size)

    random_sample = rand

    def randn(self, *size):
        return self.standard_normal(size)

    def randint(self, low, high=None, size=None, dtype=int):
        return self.integers(low=low, high=high, size=size, dtype=dtype)

    random_integers = randint

    def get_state(self):
        return self.bit_generator.state

    def set_state(self, state):
        self.bit_generator.state = state

    def seed(self, seed=None):
        self.bit_generator.state = type(self.bit_generator)(seed).state

    rand.__doc__ = np.random.rand.__doc__
    randn.__doc__ = np.random.randn.__doc__
    randint.__doc__ = np.random.randint.__doc__
    get_state.__doc__ = np.random.get_state.__doc__
    set_state.__doc__ = np.random.set_state.__doc__
    seed.__doc__ = np.random.seed.__doc__

    def __reduce__(self):
        # np.random.Generator defines __reduce__, but it's hard-coded to
        # return a Generator instead of its subclass RandomNumberGenerator.
        # We need to override it here, otherwise sampling from a Space will
        # be broken after pickling and unpickling, due to using the deprecated
        # methods defined above.
        # See: https://github.com/numpy/numpy/blob/41d37b714caa1eef72f984d529f1d40ed48ce535/numpy/random/_generator.pyx#L221-L223
        # And: https://github.com/numpy/numpy/blob/41d37b714caa1eef72f984d529f1d40ed48ce535/numpy/random/_pickle.py#L17-L37
        _, init_args, *args = np.random.Generator.__reduce__(self)
        return (RandomNumberGenerator._generator_ctor, init_args, *args)

    @staticmethod
    def _generator_ctor(bit_generator_name="MT19937"):
        # Workaround method for RandomNumberGenerator pickling, see __reduce__ above.
        # Ported from numpy.random._pickle.__generator_ctor function.
        from numpy.random._pickle import BitGenerators

        if bit_generator_name in BitGenerators:
            bit_generator = BitGenerators[bit_generator_name]
        else:
            raise ValueError(
                f"{bit_generator_name} is not a known BitGenerator module."
            )
        return RandomNumberGenerator(bit_generator())


RNG = RandomNumberGenerator

# Legacy functions


def hash_seed(seed: Optional[int] = None, max_bytes: int = 8) -> int:
    """Any given evaluation is likely to have many PRNG's active at
    once. (Most commonly, because the environment is running in
    multiple processes.) There's literature indicating that having
    linear correlations between seeds of multiple PRNG's can correlate
    the outputs:
    http://blogs.unity3d.com/2015/01/07/a-primer-on-repeatable-random-numbers/
    http://stackoverflow.com/questions/1554958/how-different-do-random-seeds-need-to-be
    http://dl.acm.org/citation.cfm?id=1276928
    Thus, for sanity we hash the seeds before using them. (This scheme
    is likely not crypto-strength, but it should be good enough to get
    rid of simple correlations.)

    Args:
    ----
        seed: None seeds from an operating system specific randomness source.
        max_bytes: Maximum number of bytes to use in the hashed seed.

    """
    if seed is None:
        seed = create_seed(max_bytes=max_bytes)
    hash = hashlib.sha512(str(seed).encode("utf8")).digest()
    return _bigint_from_bytes(hash[:max_bytes])


def create_seed(a: Optional[Union[int, str]] = None, max_bytes: int = 8) -> int:
    """Create a strong random seed. Otherwise, Python 2 would seed using
    the system time, which might be non-robust especially in the
    presence of concurrency.

    Args:
    ----
        a: None seeds from an operating system specific randomness source.
        max_bytes: Maximum number of bytes to use in the seed.

    """
    # Adapted from https://svn.python.org/projects/python/tags/r32/Lib/random.py
    if a is None:
        a = _bigint_from_bytes(os.urandom(max_bytes))
    elif isinstance(a, str):
        bt = a.encode("utf8")
        bt += hashlib.sha512(bt).digest()
        a = _bigint_from_bytes(bt[:max_bytes])
    elif isinstance(a, int):
        a = int(a % 2 ** (8 * max_bytes))
    else:
        raise Exception(f"Invalid type for seed: {type(a)} ({a})")
    return a


# TODO: don't hardcode sizeof_int here
def _bigint_from_bytes(bt: bytes) -> int:
    sizeof_int = 4
    padding = sizeof_int - len(bt) % sizeof_int
    bt += b"\0" * padding
    int_count = int(len(bt) / sizeof_int)
    unpacked = struct.unpack(f"{int_count}I", bt)
    accum = 0
    for i, val in enumerate(unpacked):
        accum += 2 ** (sizeof_int * 8 * i) * val
    return accum


def _int_list_from_bigint(bigint: int) -> List[int]:
    if bigint < 0:
        raise Exception(f"Seed must be non-negative, not {bigint}")
    elif bigint == 0:
        return [0]

    ints: List[int] = []
    while bigint > 0:
        bigint, mod = divmod(bigint, 2**32)
        ints.append(mod)
    return ints



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.models.model_summary import ModelSummary

class TestModelSummary(unittest.TestCase):
    """ModelSummary unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ModelSummary:
        """Test ModelSummary
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ModelSummary`
        """
        model = ModelSummary()
        if include_optional:
            return ModelSummary(
                id = 0,
                name = 'My Model',
                visibility = 'PUBLIC',
                description = 'A description of your model',
                creator = jaqpotpy.api.openapi.models.user.User(
                    id = '', 
                    username = '', 
                    first_name = '', 
                    last_name = '', 
                    email = '', 
                    email_verified = True, ),
                type = 'SKLEARN',
                dependent_features_length = 56,
                independent_features_length = 56,
                shared_with_organizations = [
                    jaqpotpy.api.openapi.models.organization_summary.OrganizationSummary(
                        id = 0, 
                        name = 'My Organization', )
                    ],
                created_at = '2023-01-01T12:00Z',
                updated_at = '2023-01-01T12:00:00Z'
            )
        else:
            return ModelSummary(
                id = 0,
                name = 'My Model',
                visibility = 'PUBLIC',
                type = 'SKLEARN',
                shared_with_organizations = [
                    jaqpotpy.api.openapi.models.organization_summary.OrganizationSummary(
                        id = 0, 
                        name = 'My Organization', )
                    ],
                created_at = '2023-01-01T12:00Z',
        )
        """

    def testModelSummary(self):
        """Test ModelSummary"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



from rdkit import Chem
import torch
from torch_geometric.data import Data
import json
from collections import OrderedDict


class SmilesGraphFeaturizer:
    """
    Featurizes SMILES strings into graph data suitable for graph neural networks.
    Class Attributes:
    SUPPORTED_ATOM_FEATURES (dict): Dictionary mapping atom features to their respective featurization function.
    SUPPORTED_BOND_FEATURES (dict): Dictionary mapping bond features to their respective featurization function.
    NON_ONE_HOT_ENCODED (list): List of features that should not be one-hot encoded.
    """

    SUPPORTED_ATOM_FEATURES = {
        "symbol": lambda atom: atom.GetSymbol(),
        "degree": lambda atom: atom.GetDegree(),
        "total_degree": lambda atom: atom.GetTotalDegree(),
        "formal_charge": lambda atom: atom.GetFormalCharge(),
        "num_radical_electrons": lambda atom: atom.GetNumRadicalElectrons(),
        "hybridization": lambda atom: atom.GetHybridization(),
        "is_aromatic": lambda atom: atom.GetIsAromatic(),
        "is_in_ring": lambda atom: atom.IsInRing(),
        "total_num_hs": lambda atom: atom.GetTotalNumHs(),
        "num_explicit_hs": lambda atom: atom.GetNumExplicitHs(),
        "num_implicit_hs": lambda atom: atom.GetNumImplicitHs(),
        "_ChiralityPossible": lambda atom: atom.HasProp("_ChiralityPossible"),
        "isotope": lambda atom: atom.GetIsotope(),
        "total_valence": lambda atom: atom.GetTotalValence(),
        "explicit_valence": lambda atom: atom.GetExplicitValence(),
        "implicit_valence": lambda atom: atom.GetImplicitValence(),
        "chiral_tag": lambda atom: atom.GetChiralTag(),
        "mass": lambda atom: (atom.GetMass() - 14.5275) / 9.4154,
    }
    SUPPORTED_BOND_FEATURES = {
        "bond_type": lambda bond: bond.GetBondType(),
        "is_conjugated": lambda bond: bond.GetIsConjugated(),
        "is_in_ring": lambda bond: bond.IsInRing(),
        "stereo": lambda bond: bond.GetStereo(),
    }
    NON_ONE_HOT_ENCODED = [
        "formal_charge",
        "num_radical_electrons",
        "is_aromatic",
        "_ChiralityPossible",
        "is_conjugated",
        "is_in_ring",
        "mass",
    ]

    def __init__(self, include_edge_features=True):
        """include_edge_features (bool): Whether to include edge features in the featurization."""
        self.include_edge_features = include_edge_features
        self.atom_allowable_sets = {}
        self.bond_allowable_sets = {}

    def _set_atom_allowable_sets(self, atom_allowable_sets_dict):
        """
        Sets the allowable sets for atom feature. This is used internally.
        Atom_allowable_sets_dict (dict): Dictionary of allowable sets per atom feature.
        """
        for atom_feature, allowable_set in atom_allowable_sets_dict.items():
            self.add_atom_feature(atom_feature, allowable_set)

    def _set_bond_allowable_sets(self, bond_allowable_sets_dict):
        """
        Sets the allowable sets for bond features. This is used internally.
        Bond_allowable_sets_dict (dict): Dictionary of allowable sets per bond feature.
        """
        for bond_feature, allowable_set in bond_allowable_sets_dict.items():
            self.add_bond_feature(bond_feature, allowable_set)

    def add_atom_feature(self, atom_feature, allowable_set=None):
        """
        Adds an atom feature to the featurizer. Used externally to add atom features.
        -atom_feature (str): The atom feature name to be added.
        -allowable_set (list or None): The allowable set for the atom feature.
        'UNK' is used as a special keyword. When an atom feature is encoded and an unknown
        value (not in the allowable set) is encountered during featurization, it will be encoded as 'UNK'.
        This allows handling of unexpected values by mapping them to a default or placeholder category.
        """
        # Error handling when user input feature name does not match self.SUPPORTED_ATOM_FEATURES
        if atom_feature not in self.SUPPORTED_ATOM_FEATURES.keys():
            raise ValueError(f"Unsupported atom feature '{atom_feature}'")
        # Placeholder for the feature to contain a single element for non one hot encoded integers
        if atom_feature in self.NON_ONE_HOT_ENCODED:
            self.atom_allowable_sets[atom_feature] = None
        else:
            # If user does not provide allowable set, use default
            if allowable_set is None:
                allowable_set = self.get_default_atom_allowable_set(atom_feature)
            # Error handling for user input allowable set data type
            elif not isinstance(allowable_set, (list, tuple)):
                raise TypeError(
                    "Input dictionary must have values of type list, tuple or None."
                )
            # Creates a dict where key is the string name of the feature and values are the allowable set
            # This is if user input provides allowable set based on RDKit
            else:
                self.atom_allowable_sets[atom_feature] = list(allowable_set)

    def add_bond_feature(self, bond_feature, allowable_set=None):
        """
        Adds a bond feature to the featurizer. Used externally to add bond features.
        -bond_feature (str): The bond feature to add.
        -allowable_set (list or None): The allowable set for the bond feature.
        'UNK' is used as a special keyword. When a bond feature is encoded and an unknown
        value (not in the allowable set) is encountered during featurization, it will be encoded as 'UNK'.
        This allows handling of unexpected values by mapping them to a default or placeholder category.
        """
        # Error handling when user input feature name does not match self.SUPPORTED_ATOM_FEATURES
        if bond_feature not in self.SUPPORTED_BOND_FEATURES.keys():
            raise ValueError(f"Unsupported bond feature '{bond_feature}'")
        if bond_feature in self.NON_ONE_HOT_ENCODED:
            # Placeholder for the feature to contain a single element for non one hot encoded integers
            self.bond_allowable_sets[bond_feature] = None
        else:
            # If user does not provide allowable set, use default
            if allowable_set is None:
                allowable_set = self.get_default_bond_allowable_set(bond_feature)
            # Error handling for user input allowable set data type
            elif not isinstance(allowable_set, (list, tuple)):
                raise TypeError(
                    "Input dictionary must have values of type list, tuple or None."
                )
            # Creates a dict where key is the string name of the feature and values are the allowable set
            # This is if user input provides allowable set based on RDKit
            else:
                self.bond_allowable_sets[bond_feature] = list(allowable_set)

    def get_default_atom_allowable_set(self, atom_feature):
        """Returns the default allowable set for an atom feature."""
        match atom_feature:
            case "symbol":
                return [
                    "C",
                    "O",
                    "N",
                    "Cl",
                    "S",
                    "F",
                    "Na",
                    "P",
                    "Br",
                    "Si",
                    "K",
                    "Sn",
                    "UNK",
                ]
            case "degree":
                return [0, 1, 2, 3, 4]
            case "hybridization":
                return [
                    Chem.rdchem.HybridizationType.S,
                    Chem.rdchem.HybridizationType.SP2,
                    Chem.rdchem.HybridizationType.SP3,
                    Chem.rdchem.HybridizationType.SP3D,
                    Chem.rdchem.HybridizationType.SP3D2,
                ]
            case "total_num_hs":
                return [0, 1, 2, 3, 4]
            case "implicit_valence":
                return [0, 1, 2, 3, 4]
            case "explicit_valence":
                return [4, 2, 3, 1, 0, "UNK"]
            case _:
                raise ValueError(
                    f"No default allowable set for atom characteristic '{atom_feature}'. You must set your own allowable set."
                )

    def get_default_bond_allowable_set(self, bond_feature):
        """Returns the default allowable set for a bond feature."""
        match bond_feature:
            case "bond_type":
                return [
                    Chem.rdchem.BondType.SINGLE,
                    Chem.rdchem.BondType.DOUBLE,
                    Chem.rdchem.BondType.TRIPLE,
                    Chem.rdchem.BondType.AROMATIC,
                ]
            case _:
                raise ValueError(
                    f"No default allowable set for bond characteristic '{bond_feature}'. You must set your own allowable set."
                )

    def sort_allowable_sets(self):
        """Sorts the allowable sets for atom and bond features."""
        self.atom_allowable_sets = OrderedDict(sorted(self.atom_allowable_sets.items()))
        self.bond_allowable_sets = OrderedDict(sorted(self.bond_allowable_sets.items()))

        return self

    def set_default_config(self):
        """Sets the default configuration for the featurizer."""
        atom_allowable_sets = {
            "symbol": self.get_default_atom_allowable_set("symbol"),
            "degree": self.get_default_atom_allowable_set("degree"),
            "hybridization": self.get_default_atom_allowable_set("hybridization"),
            "total_num_hs": self.get_default_atom_allowable_set("total_num_hs"),
            "is_aromatic": None,
            "formal_charge": None,
        }

        bond_allowable_sets = {
            "bond_type": self.get_default_bond_allowable_set("bond_type"),
            "is_conjugated": None,
            "is_in_ring": None,
        }
        self._set_atom_allowable_sets(atom_allowable_sets)
        if self.include_edge_features:
            self._set_bond_allowable_sets(bond_allowable_sets)

    def _extract_molecular_features(self, mol):
        """Extracts molecular features from a molecule. Returns the atom and bond (optionally) features for a molecule."""
        mol_atom_features = []
        for atom in mol.GetAtoms():
            mol_atom_features.append(self._atom_features(atom))
        mol_atom_features = torch.tensor(mol_atom_features, dtype=torch.float32)
        if not self.include_edge_features:
            return mol_atom_features, None
        mol_bond_features = []
        for bond in mol.GetBonds():
            mol_bond_features.append(self._bond_features(bond))
            mol_bond_features.append(self._bond_features(bond))
        mol_bond_features = torch.tensor(mol_bond_features, dtype=torch.float32)
        return mol_atom_features, mol_bond_features

    def _atom_features(self, atom):
        """Extracts features for an atom. Used internally."""
        feats = []
        for feature in self.atom_allowable_sets.keys():
            # Contains the rdkit function for feature calculation
            property_getter = self.SUPPORTED_ATOM_FEATURES[feature]
            feat = property_getter(atom)
            if feature in self.NON_ONE_HOT_ENCODED:
                feats.append(feat)
            else:
                allowable_set = self.atom_allowable_sets[feature]
                if "UNK" in allowable_set:
                    one_hot_encoded_feat = self._one_of_k_encoding_unk(
                        feat, allowable_set
                    )
                else:
                    one_hot_encoded_feat = self._one_of_k_encoding(
                        feat, allowable_set, feature_name=feature
                    )
                feats.extend(one_hot_encoded_feat)
        return feats

    def _bond_features(self, bond):
        """Extracts features for a bond. Used internally."""
        feats = []
        for feature in self.bond_allowable_sets.keys():
            property_getter = self.SUPPORTED_BOND_FEATURES[feature]
            feat = property_getter(bond)

            if feature in self.NON_ONE_HOT_ENCODED:
                feats.append(feat)
            else:
                allowable_set = self.bond_allowable_sets[feature]
                if "UNK" in allowable_set:
                    one_hot_encoded_feat = self._one_of_k_encoding_unk(
                        feat, allowable_set
                    )
                else:
                    one_hot_encoded_feat = self._one_of_k_encoding(
                        feat, allowable_set, feature_name=feature
                    )
                feats.extend(one_hot_encoded_feat)
        return feats

    def _one_of_k_encoding(self, x, allowable_set, feature_name=None):
        """One-hot encodes a value based on an allowable set."""
        if x not in allowable_set:
            feature_text = f"{feature_name} " if feature_name else ""
            print(feature_text)
        return [x == s for s in allowable_set]

    def _one_of_k_encoding_unk(self, x, allowable_set):
        """One-hot encodes a value based on an allowable set, mapping unseen values to 'UNK'."""
        if x not in allowable_set:
            x = "UNK"
        return [x == s for s in allowable_set]

    def _adjacency_matrix(self, mol):
        """Computes the adjacency matrix for a molecule. Used internally"""
        ix1, ix2 = [], []
        for bond in mol.GetBonds():
            start, end = bond.GetBeginAtomIdx(), bond.GetEndAtomIdx()
            ix1 += [start, end]
            ix2 += [end, start]
        adj_norm = torch.asarray(
            [ix1, ix2], dtype=torch.int64
        )  # Needs to be in COO Format
        return adj_norm

    def get_num_node_features(self):
        """Returns the number of node features."""
        num = 0
        for value in self.atom_allowable_sets.values():
            if value is None:
                num += 1
            else:
                num += len(value)
        return num

    def get_num_edge_features(self):
        """Returns the number of edge features"""
        num = 0
        for value in self.bond_allowable_sets.values():
            if value is None:
                num += 1
            else:
                num += len(value)
        return num

    def __call__(self, *args, **kwargs):
        """Featurizes the input data"""
        return self.featurize(*args, **kwargs)

    def featurize(self, sm, y=None):
        """Featurizes a SMILES string into graph data. Returns torch_geometric.data.Data object."""
        mol = Chem.MolFromSmiles(sm)
        adjacency_matrix = self._adjacency_matrix(mol)
        mol_atom_features, mol_bond_features = self._extract_molecular_features(mol)
        return Data(
            x=mol_atom_features,
            edge_index=adjacency_matrix,
            edge_attr=mol_bond_features,
            y=y,
            smiles=sm,
        )

    def get_dict(self):
        """Creates a json configuration that will be sent to database for inference"""
        config_dict = {
            "include_edge_features": self.include_edge_features,
            "atom_allowable_sets": self.atom_allowable_sets,
            "bond_allowable_sets": self.bond_allowable_sets,
        }
        return config_dict

    def load_dict(self, feat_dict):
        """Loads a configuration from a json dict. Mainly used in jaqpotpy-inference"""
        self.include_edge_features = feat_dict["include_edge_features"]
        self._set_atom_allowable_sets(feat_dict["atom_allowable_sets"])
        self._set_bond_allowable_sets(feat_dict["bond_allowable_sets"])
        return self

    def __repr__(self):
        """Representation of the featurizer"""
        attributes = {
            "atom_allowable_sets": self.atom_allowable_sets,
            "bond_allowable_sets": self.bond_allowable_sets,
        }
        return type(self).__name__ + "(" + json.dumps(attributes, indent=4) + ")"

    def __str__(self):
        """Returns the string representation of the featurizer."""
        return self.__repr__()



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.models.model import Model

class TestModel(unittest.TestCase):
    """Model unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> Model:
        """Test Model
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `Model`
        """
        model = Model()
        if include_optional:
            return Model(
                id = 0,
                meta = {
                    'key' : None
                    },
                name = 'My Model',
                description = 'A description of your model',
                type = 'SKLEARN',
                jaqpotpy_version = '1.0.0',
                libraries = [
                    jaqpotpy.api.openapi.models.library.Library(
                        id = 56, 
                        name = 'Library Name', 
                        version = '1.24.0', 
                        created_at = '2023-01-01T12:00Z', 
                        updated_at = '2023-01-01T12:00:00Z', )
                    ],
                dependent_features = [
                    jaqpotpy.api.openapi.models.feature.Feature(
                        id = 1, 
                        meta = {
                            'key' : None
                            }, 
                        key = 'feature-key', 
                        name = 'A feature name', 
                        units = 'A feature unit', 
                        description = '', 
                        feature_type = 'FLOAT', 
                        feature_dependency = 'DEPENDENT', 
                        visible = True, 
                        possible_values = [
                            jaqpotpy.api.openapi.models.feature_possible_value.FeaturePossibleValue(
                                key = 'value', 
                                value = 'value', )
                            ], 
                        created_at = '2023-01-01T12:00Z', 
                        updated_at = '2023-01-01T12:00:00Z', )
                    ],
                independent_features = [
                    jaqpotpy.api.openapi.models.feature.Feature(
                        id = 1, 
                        meta = {
                            'key' : None
                            }, 
                        key = 'feature-key', 
                        name = 'A feature name', 
                        units = 'A feature unit', 
                        description = '', 
                        feature_type = 'FLOAT', 
                        feature_dependency = 'DEPENDENT', 
                        visible = True, 
                        possible_values = [
                            jaqpotpy.api.openapi.models.feature_possible_value.FeaturePossibleValue(
                                key = 'value', 
                                value = 'value', )
                            ], 
                        created_at = '2023-01-01T12:00Z', 
                        updated_at = '2023-01-01T12:00:00Z', )
                    ],
                shared_with_organizations = [
                    jaqpotpy.api.openapi.models.organization.Organization(
                        id = 56, 
                        name = 'my-awesome-org', 
                        creator = jaqpotpy.api.openapi.models.user.User(
                            id = '', 
                            username = '', 
                            first_name = '', 
                            last_name = '', 
                            email = '', 
                            email_verified = True, ), 
                        visibility = 'PUBLIC', 
                        description = 'An awesome organization for managing models.', 
                        organization_members = [
                            jaqpotpy.api.openapi.models.organization_user.OrganizationUser(
                                id = 56, 
                                user_id = '', 
                                username = '', 
                                email = '', 
                                association_type = 'ADMIN', )
                            ], 
                        contact_email = 'contact@my-awesome-org.com', 
                        contact_phone = '+1234567890', 
                        website = 'http://www.my-awesome-org.com', 
                        address = '123 Organization St., City, Country', 
                        can_edit = True, 
                        is_member = True, 
                        created_at = '', 
                        updated_at = '', )
                    ],
                visibility = 'PUBLIC',
                task = 'REGRESSION',
                raw_model = 'YQ==',
                creator = jaqpotpy.api.openapi.models.user.User(
                    id = '', 
                    username = '', 
                    first_name = '', 
                    last_name = '', 
                    email = '', 
                    email_verified = True, ),
                can_edit = True,
                is_admin = True,
                tags = '',
                legacy_prediction_service = '',
                extra_config = jaqpotpy.api.openapi.models.model_extra_config.ModelExtraConfig(
                    torch_config = {
                        'key' : None
                        }, 
                    preprocessors = [
                        jaqpotpy.api.openapi.models.transformer.Transformer(
                            name = 'StandardScaler', 
                            config = {
                                'key' : None
                                }, )
                        ], 
                    featurizers = [
                        jaqpotpy.api.openapi.models.transformer.Transformer(
                            name = 'StandardScaler', 
                            config = {
                                'key' : None
                                }, )
                        ], 
                    doa = [
                        
                        ], ),
                created_at = '2023-01-01T12:00Z',
                updated_at = '2023-01-01T12:00:00Z'
            )
        else:
            return Model(
                name = 'My Model',
                type = 'SKLEARN',
                jaqpotpy_version = '1.0.0',
                libraries = [
                    jaqpotpy.api.openapi.models.library.Library(
                        id = 56, 
                        name = 'Library Name', 
                        version = '1.24.0', 
                        created_at = '2023-01-01T12:00Z', 
                        updated_at = '2023-01-01T12:00:00Z', )
                    ],
                dependent_features = [
                    jaqpotpy.api.openapi.models.feature.Feature(
                        id = 1, 
                        meta = {
                            'key' : None
                            }, 
                        key = 'feature-key', 
                        name = 'A feature name', 
                        units = 'A feature unit', 
                        description = '', 
                        feature_type = 'FLOAT', 
                        feature_dependency = 'DEPENDENT', 
                        visible = True, 
                        possible_values = [
                            jaqpotpy.api.openapi.models.feature_possible_value.FeaturePossibleValue(
                                key = 'value', 
                                value = 'value', )
                            ], 
                        created_at = '2023-01-01T12:00Z', 
                        updated_at = '2023-01-01T12:00:00Z', )
                    ],
                independent_features = [
                    jaqpotpy.api.openapi.models.feature.Feature(
                        id = 1, 
                        meta = {
                            'key' : None
                            }, 
                        key = 'feature-key', 
                        name = 'A feature name', 
                        units = 'A feature unit', 
                        description = '', 
                        feature_type = 'FLOAT', 
                        feature_dependency = 'DEPENDENT', 
                        visible = True, 
                        possible_values = [
                            jaqpotpy.api.openapi.models.feature_possible_value.FeaturePossibleValue(
                                key = 'value', 
                                value = 'value', )
                            ], 
                        created_at = '2023-01-01T12:00Z', 
                        updated_at = '2023-01-01T12:00:00Z', )
                    ],
                visibility = 'PUBLIC',
                task = 'REGRESSION',
                raw_model = 'YQ==',
        )
        """

    def testModel(self):
        """Test Model"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.models.library import Library

class TestLibrary(unittest.TestCase):
    """Library unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> Library:
        """Test Library
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `Library`
        """
        model = Library()
        if include_optional:
            return Library(
                id = 56,
                name = 'Library Name',
                version = '1.24.0',
                created_at = '2023-01-01T12:00Z',
                updated_at = '2023-01-01T12:00:00Z'
            )
        else:
            return Library(
                name = 'Library Name',
                version = '1.24.0',
        )
        """

    def testLibrary(self):
        """Test Library"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



from abc import ABC, abstractmethod
import pandas as pd
import numpy as np
from typing import Iterable, Any, Union


class DOA(ABC):
    """Abstract class for DOA methods"""

    @property
    def __name__(self):
        return NotImplementedError

    @property
    def doa_new(self):
        return self._doa

    @doa_new.setter
    def doa_new(self, value):
        self._doa = value

    @property
    def in_doa(self):
        return self._in_doa

    @in_doa.setter
    def in_doa(self, value):
        self._in_doa = value

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, value):
        self._data = value

    @abstractmethod
    def fit(self, X: np.array):
        raise NotImplementedError

    @abstractmethod
    def predict(self, data: Iterable[Any]) -> Iterable[Any]:
        raise NotImplementedError

    @abstractmethod
    def get_attributes(self):
        raise NotImplementedError


class Leverage(DOA):
    """Implements DOA method leverage.
    Initialized upon training data and holds the doa matrix and the threshold 'A' value.
    Calculates the DOA for a new instance of data or array of data.
    """

    _doa = []
    _in_doa = []

    @property
    def __name__(self):
        return "LeverageDoa"

    def __init__(self) -> None:
        # self._scaler: BaseEstimator = scaler
        self._data: Union[np.array, pd.DataFrame] = None
        self._doa_matrix = None
        self._h_star = None
        self._doa_attributes = None

    def __getitem__(self, key):
        return key

    @property
    def doa_matrix(self):
        return self._doa_matrix

    @doa_matrix.setter
    def doa_matrix(self, value):
        self._doa_matrix = value

    @property
    def h_star(self):
        return self._h_star

    @h_star.setter
    def h_star(self, value):
        self._h_star = value

    def calculate_threshold(self):
        shape = self._data.shape
        h_star = (3 * (shape[1] + 1)) / shape[0]
        self._h_star = h_star

    def calculate_matrix(self):
        x_T = self._data.transpose()
        x_out = x_T.dot(self._data)
        self._doa_matrix = np.linalg.pinv(x_out)

    def fit(self, X: Union[np.array, pd.DataFrame]):
        self._data = self._validate_input(X)
        self.calculate_matrix()
        self.calculate_threshold()
        self._doa_attributes = self.get_attributes()

    def predict(self, new_data: Union[np.array, pd.DataFrame]) -> Iterable[Any]:
        new_data = self._validate_input(new_data)
        doaAll = []
        self._doa = []
        self._in_doa = []
        for nd in new_data:
            d1 = np.dot(nd, self.doa_matrix)
            ndt = np.transpose(nd)
            d2 = np.dot(d1, ndt)
            if d2 < self._h_star:
                in_ad = True
            else:
                in_ad = False
            self._doa.append(d2)
            self._in_doa.append(in_ad)
            doa = {"DOA": d2, "A": self._h_star, "in_doa": in_ad}
            doaAll.append(doa)
        return doaAll

    def _validate_input(self, data: Union[np.array, pd.DataFrame]):
        if isinstance(data, pd.DataFrame):
            return data.to_numpy()
        else:
            return data

    def get_attributes(self):
        return {"doa_matrix": self.doa_matrix, "h_star": self.h_star}


class MeanVar(DOA):
    """Implements Mean and Variance domain of applicability .
    Initialized upon training data and holds the doa mean and the variance of the data.
    Calculates the mean and variance for a new instance of data or array of data and decides if in AD.
    """

    _doa = []
    _in_doa = []

    @property
    def __name__(self):
        return "MeanVar"

    def __init__(self) -> None:
        self._data: np.array = None
        self._bounds = None
        self._doa_attributes = None

    def fit(self, X: np.array):
        self._data = X
        list_m_var = []
        for i in range(self._data.shape[1]):
            list_m_var.append(
                [
                    np.mean(self._data[:, i]),
                    np.std(self._data[:, i]),
                    np.var(self._data[:, i]),
                ]
            )
        self._bounds = np.array(list_m_var)
        self._doa_attributes = self.get_attributes()

    def predict(self, new_data: np.array) -> Iterable[Any]:
        doaAll = []
        self._doa = []
        self._in_doa = []
        in_doa = True
        for nd in new_data:
            for index, row in enumerate(nd):
                bounds = self._bounds[index]
                bounds_data = [bounds[0] - 4 * bounds[1], bounds[0] + 4 * bounds[1]]
                if row >= bounds_data[0] and row <= bounds_data[1]:
                    continue
                else:
                    in_doa = False
            doa = {"in_doa": in_doa}
            doaAll.append(doa)
            self._doa.append(new_data)
            self._in_doa.append(in_doa)
        return doaAll

    def get_attributes(self):
        return {"mean_var": self._bounds}


class BoundingBox(DOA):
    _doa = []
    _in_doa = []

    @property
    def __name__(self):
        return "BoundingBox"

    def __init__(self) -> None:
        self._data: np.array = None
        self._bounding_box = None
        self._doa_attributes = None

    def fit(self, X: np.array):
        self._data = X
        list_m_var = []
        for i in range(self._data.shape[1]):
            list_m_var.append([np.min(self._data[:, i]), np.max(self._data[:, i])])
        self._bounding_box = np.array(list_m_var)
        self._doa_attributes = self.get_attributes()

    def predict(self, new_data: np.array) -> Iterable[Any]:
        doaAll = []
        self._doa = []
        self._in_doa = []
        in_doa = True
        for nd in new_data:
            for index, row in enumerate(nd):
                bounds = self._bounding_box[index]
                bounds_data = [bounds[0], bounds[1]]
                if row >= bounds_data[0] and row <= bounds_data[1]:
                    continue
                else:
                    in_doa = False
            doa = {"in_doa": in_doa}
            doaAll.append(doa)
            self._doa.append(new_data)
            self._in_doa.append(in_doa)
        return doaAll

    def get_attributes(self):
        return {"bounding_box": self._bounding_box}



from torch.utils.data import Dataset
from jaqpotpy.descriptors.graph.graph_featurizer import SmilesGraphFeaturizer
from typing import Any, Optional


class SmilesGraphDataset(Dataset):
    """
    A PyTorch Dataset class for handling SMILES strings as graphs.
    This class overrides `__getitem__` and `__len__` (check source code for methods' docstrings).
    Attributes:
    smiles (list): A list of SMILES strings.
    y (list, optional): A list of target values.
    featurizer (SmilesGraphFeaturizer): The object to transform SMILES strings into graph representations.
    precomputed_features (list, optional): A list of precomputed features. If precompute_featurization() is not called, this attribute remains None.
    """

    def __init__(
        self,
        smiles: list = None,
        y: Optional[list] = None,
        featurizer: Optional[SmilesGraphFeaturizer] = None,
    ):
        """The SmilesGraphDataset constructor."""
        super().__init__()
        self.smiles = smiles
        self.y = y
        # For the moment there is only one Featurizer for Graph.
        if featurizer:
            self.featurizer = featurizer
            self.featurizer.sort_allowable_sets()
        else:
            self.featurizer = SmilesGraphFeaturizer()
            # Default node, edge features in case of not specifying dataset
            self.featurizer.set_default_config()
            self.featurizer.sort_allowable_sets()
        self.precomputed_features = None

    def precompute_featurization(self):
        """Precomputes the featurization of the dataset before being accessed by __getitem__"""
        if self.y:
            self.precomputed_features = [
                self.featurizer(sm, y) for sm, y, in zip(self.smiles, self.y)
            ]
        else:
            self.precomputed_features = [self.featurizer(sm) for sm in self.smiles]

    def get_num_node_features(self):
        """Returns the number of node features."""
        return len(self.get_atom_feature_labels())

    def get_num_edge_features(self):
        """Returns the number of edge features."""
        return len(self.get_bond_feature_labels())

    def __getitem__(self, idx):
        """
        Retrieves the featurized graph Data object and target value for a given index.
        Args:
            idx (int): Index of the data to retrieve.
        Returns:
            torch_geometric.data.Data: A torch_geometric.data.Data object for this single sample containing:
                - x (torch.Tensor): Node feature matrix with shape [num_nodes, num_node_features].
                - edge_index (LongTensor): Graph connectivity in COO format with shape [2, num_edges].
                - edge_attr (torch.Tensor, optional): Edge feature matrix with shape [num_edges, num_edge_features].
                - y (float): Graph-level ground-truth label.
                - smiles (str): The SMILES string corresponding to the particular sample.
        """
        if self.precomputed_features:
            return self.precomputed_features[idx]
        sm = self.smiles[idx]
        y = self.y[idx] if self.y else None
        return self.featurizer(sm, y)

    def __len__(self):
        """
        __len__ functionality is important for the DataLoader to determine batching,
        shuffling and iterating over the dataset.
        """
        return len(self.smiles)

    def __repr__(self) -> str:
        """Official string representation of the Dataset Object"""
        return self.__class__.__name__



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.models.get_datasets200_response import GetDatasets200Response

class TestGetDatasets200Response(unittest.TestCase):
    """GetDatasets200Response unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> GetDatasets200Response:
        """Test GetDatasets200Response
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `GetDatasets200Response`
        """
        model = GetDatasets200Response()
        if include_optional:
            return GetDatasets200Response(
                content = [
                    jaqpotpy.api.openapi.models.dataset.Dataset(
                        id = 1, 
                        type = 'PREDICTION', 
                        entry_type = 'ARRAY', 
                        input = [
                            null
                            ], 
                        result = [
                            null
                            ], 
                        status = 'CREATED', 
                        failure_reason = '', 
                        user_id = '', 
                        model_id = 56, 
                        model_name = '', 
                        executed_at = '', 
                        execution_finished_at = '', 
                        created_at = '', 
                        updated_at = '', )
                    ],
                total_elements = 56,
                total_pages = 56,
                page_size = 56,
                page_number = 56
            )
        else:
            return GetDatasets200Response(
        )
        """

    def testGetDatasets200Response(self):
        """Test GetDatasets200Response"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.api.organization_invitation_api import OrganizationInvitationApi


class TestOrganizationInvitationApi(unittest.TestCase):
    """OrganizationInvitationApi unit test stubs"""

    def setUp(self) -> None:
        self.api = OrganizationInvitationApi()

    def tearDown(self) -> None:
        pass

    def test_create_invitations(self) -> None:
        """Test case for create_invitations

        Create new invitations for an organization
        """
        pass

    def test_get_all_invitations(self) -> None:
        """Test case for get_all_invitations

        Get all invitations for an organization
        """
        pass

    def test_get_invitation(self) -> None:
        """Test case for get_invitation

        Get the status of an invitation
        """
        pass

    def test_resend_invitation(self) -> None:
        """Test case for resend_invitation

        Resend an invitation email
        """
        pass

    def test_update_invitation(self) -> None:
        """Test case for update_invitation

        Update the status of an invitation
        """
        pass


if __name__ == '__main__':
    unittest.main()



from jaqpotpy.api.jaqpot_api_http_client import JaqpotApiHttpClient


class JaqpotApiHttpClientBuilder:
    def __init__(self, host):
        self.http_client = JaqpotApiHttpClient(host=host)  
        
    def build_with_access_token(self, access_token):
        self.http_client.set_access_token(access_token)
        return self
    
    def build_with_api_keys(self, client_key, client_secret):
        self.http_client.set_default_header('X-Api-Key', client_key)
        self.http_client.set_default_header('X-Api-Secret', client_secret)
        return self
        
    def build(self):
        return self.http_client
        



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.api.organization_api import OrganizationApi


class TestOrganizationApi(unittest.TestCase):
    """OrganizationApi unit test stubs"""

    def setUp(self) -> None:
        self.api = OrganizationApi()

    def tearDown(self) -> None:
        pass

    def test_create_organization(self) -> None:
        """Test case for create_organization

        Create a new organization
        """
        pass

    def test_get_all_organizations_by_user(self) -> None:
        """Test case for get_all_organizations_by_user

        Get all user organizations
        """
        pass

    def test_get_all_organizations_for_user(self) -> None:
        """Test case for get_all_organizations_for_user

        Get all organizations for a specific user
        """
        pass

    def test_get_organization_by_name(self) -> None:
        """Test case for get_organization_by_name

        Get organization by name
        """
        pass

    def test_partial_update_organization(self) -> None:
        """Test case for partial_update_organization

        Partially update an existing organization
        """
        pass


if __name__ == '__main__':
    unittest.main()



"""Utilities for constructing node features or bond features.
Some functions are based on chainer-chemistry or dgl-lifesci.
Repositories:
- https://github.com/chainer/chainer-chemistry
- https://github.com/awslabs/dgl-lifesci
"""

import os
import logging
from typing import List, Union, Tuple

import numpy as np

from jaqpotpy.utils.types import RDKitAtom, RDKitBond, RDKitMol

logger = logging.getLogger(__name__)

DEFAULT_ATOM_TYPE_SET = [
    "C",
    "N",
    "O",
    "F",
    "P",
    "S",
    "Cl",
    "Br",
    "I",
]
DEFAULT_HYBRIDIZATION_SET = ["SP", "SP2", "SP3"]
DEFAULT_TOTAL_NUM_Hs_SET = [0, 1, 2, 3, 4]
DEFAULT_FORMAL_CHARGE_SET = [-2, -1, 0, 1, 2]
DEFAULT_TOTAL_DEGREE_SET = [0, 1, 2, 3, 4, 5]
DEFAULT_RING_SIZE_SET = [3, 4, 5, 6, 7, 8]
DEFAULT_BOND_TYPE_SET = ["SINGLE", "DOUBLE", "TRIPLE", "AROMATIC"]
DEFAULT_BOND_STEREO_SET = ["STEREONONE", "STEREOANY", "STEREOZ", "STEREOE"]
DEFAULT_GRAPH_DISTANCE_SET = [1, 2, 3, 4, 5, 6, 7]
DEFAULT_ATOM_IMPLICIT_VALENCE_SET = [0, 1, 2, 3, 4, 5, 6]
DEFAULT_ATOM_EXPLICIT_VALENCE_SET = [1, 2, 3, 4, 5, 6]


class _ChemicalFeaturesFactory:
    """This is a singleton class for RDKit base features."""

    _instance = None

    @classmethod
    def get_instance(cls):
        try:
            from rdkit import RDConfig
            from rdkit.Chem import ChemicalFeatures
        except ModuleNotFoundError:
            raise ImportError("This class requires RDKit to be installed.")

        if not cls._instance:
            fdefName = os.path.join(RDConfig.RDDataDir, "BaseFeatures.fdef")
            cls._instance = ChemicalFeatures.BuildFeatureFactory(fdefName)
        return cls._instance


def one_hot_encode(
    val: Union[int, str],
    allowable_set: Union[List[str], List[int]],
    include_unknown_set: bool = False,
) -> List[float]:
    """One hot encoder for elements of a provided set.

    Examples
    --------
    >>> one_hot_encode("a", ["a", "b", "c"])
    [1.0, 0.0, 0.0]
    >>> one_hot_encode(2, [0, 1, 2])
    [0.0, 0.0, 1.0]
    >>> one_hot_encode(3, [0, 1, 2])
    [0.0, 0.0, 0.0]
    >>> one_hot_encode(3, [0, 1, 2], True)
    [0.0, 0.0, 0.0, 1.0]

    Parameters
    ----------
    val: int or str
      The value must be present in `allowable_set`.
    allowable_set: List[int] or List[str]
      List of allowable quantities.
    include_unknown_set: bool, default False
      If true, the index of all values not in `allowable_set` is `len(allowable_set)`.

    Returns
    -------
    List[float]
      An one-hot vector of val.
      If `include_unknown_set` is False, the length is `len(allowable_set)`.
      If `include_unknown_set` is True, the length is `len(allowable_set) + 1`.

    Raises
    ------
    ValueError
      If include_unknown_set is False and `val` is not in `allowable_set`.

    """
    if include_unknown_set is False:
        if val not in allowable_set:
            logger.info(
                "input {0} not in allowable set {1}:".format(val, allowable_set)
            )

    # init an one-hot vector
    if include_unknown_set is False:
        one_hot_legnth = len(allowable_set)
    else:
        one_hot_legnth = len(allowable_set) + 1
    one_hot = [0.0 for _ in range(one_hot_legnth)]

    try:
        one_hot[allowable_set.index(val)] = 1.0  # type: ignore
    except:
        if include_unknown_set:
            # If include_unknown_set is True, set the last index is 1.
            one_hot[-1] = 1.0
        else:
            pass
    return one_hot


#################################################################
# atom (node) featurization
#################################################################


def get_atom_type_one_hot(
    atom: RDKitAtom,
    allowable_set: List[str] = DEFAULT_ATOM_TYPE_SET,
    include_unknown_set: bool = True,
) -> List[float]:
    """Get an one-hot feature of an atom type.

    Parameters
    ----------
    atom: rdkit.Chem.rdchem.Atom
      RDKit atom object
    allowable_set: List[str]
      The atom types to consider. The default set is
      `["C", "N", "O", "F", "P", "S", "Cl", "Br", "I"]`.
    include_unknown_set: bool, default True
      If true, the index of all atom not in `allowable_set` is `len(allowable_set)`.

    Returns
    -------
    List[float]
      An one-hot vector of atom types.
      If `include_unknown_set` is False, the length is `len(allowable_set)`.
      If `include_unknown_set` is True, the length is `len(allowable_set) + 1`.

    """
    return one_hot_encode(atom.GetSymbol(), allowable_set, include_unknown_set)


def construct_hydrogen_bonding_info(mol: RDKitMol) -> List[Tuple[int, str]]:
    """Construct hydrogen bonding infos about a molecule.

    Parameters
    ----------
    mol: rdkit.Chem.rdchem.Mol
      RDKit mol object

    Returns
    -------
    List[Tuple[int, str]]
      A list of tuple `(atom_index, hydrogen_bonding_type)`.
      The `hydrogen_bonding_type` value is "Acceptor" or "Donor".

    """
    factory = _ChemicalFeaturesFactory.get_instance()
    feats = factory.GetFeaturesForMol(mol)
    hydrogen_bonding = []
    for f in feats:
        hydrogen_bonding.append((f.GetAtomIds()[0], f.GetFamily()))
    return hydrogen_bonding


def get_atom_num_radical_electrons(atom: RDKitAtom):
    """Get the number of radical electrons for an atom.

    Parameters
    ----------
    atom : rdkit.Chem.rdchem.Atom
        RDKit atom instance.

    Returns
    -------
    list
        List containing one int only.

    See Also
    --------
    atom_num_radical_electrons_one_hot

    """
    return [atom.GetNumRadicalElectrons()]


def get_atom_is_aromatic(atom: RDKitAtom):
    """Get whether the atom is aromatic.

    Parameters
    ----------
    atom : rdkit.Chem.rdchem.Atom
        RDKit atom instance.

    Returns
    -------
    list
        List containing one bool only.

    See Also
    --------
    atom_is_aromatic_one_hot

    """
    return [atom.GetIsAromatic()]


def get_atom_is_chiral_center(atom: RDKitAtom):
    """Get whether the atom is chiral center

    Parameters
    ----------
    atom : rdkit.Chem.rdchem.Atom
        RDKit atom instance.

    Returns
    -------
    list
        List containing one bool only.

    """
    return [atom.HasProp("_ChiralityPossible")]


def get_atom_hydrogen_bonding_one_hot(
    atom: RDKitAtom, hydrogen_bonding: List[Tuple[int, str]]
) -> List[float]:
    """Get an one-hot feat about whether an atom accepts electrons or donates electrons.

    Parameters
    ----------
    atom: rdkit.Chem.rdchem.Atom
      RDKit atom object
    hydrogen_bonding: List[Tuple[int, str]]
      The return value of `construct_hydrogen_bonding_info`.
      The value is a list of tuple `(atom_index, hydrogen_bonding)` like (1, "Acceptor").

    Returns
    -------
    List[float]
      A one-hot vector of the ring size type. The first element
      indicates "Donor", and the second element indicates "Acceptor".

    """
    one_hot = [0.0, 0.0]
    atom_idx = atom.GetIdx()
    for hydrogen_bonding_tuple in hydrogen_bonding:
        if hydrogen_bonding_tuple[0] == atom_idx:
            if hydrogen_bonding_tuple[1] == "Donor":
                one_hot[0] = 1.0
            elif hydrogen_bonding_tuple[1] == "Acceptor":
                one_hot[1] = 1.0
    return one_hot


def get_atom_is_in_aromatic_one_hot(atom: RDKitAtom) -> List[float]:
    """Get ans one-hot feature about whether an atom is in aromatic system or not.

    Parameters
    ----------
    atom: rdkit.Chem.rdchem.Atom
      RDKit atom object

    Returns
    -------
    List[float]
      A vector of whether an atom is in aromatic system or not.

    """
    return [float(atom.GetIsAromatic())]


def get_atom_hybridization_one_hot(
    atom: RDKitAtom,
    allowable_set: List[str] = DEFAULT_HYBRIDIZATION_SET,
    include_unknown_set: bool = False,
) -> List[float]:
    """Get an one-hot feature of hybridization type.

    Parameters
    ----------
    atom: rdkit.Chem.rdchem.Atom
      RDKit atom object
    allowable_set: List[str]
      The hybridization types to consider. The default set is `["SP", "SP2", "SP3"]`
    include_unknown_set: bool, default False
      If true, the index of all types not in `allowable_set` is `len(allowable_set)`.

    Returns
    -------
    List[float]
      An one-hot vector of the hybridization type.
      If `include_unknown_set` is False, the length is `len(allowable_set)`.
      If `include_unknown_set` is True, the length is `len(allowable_set) + 1`.

    """
    return one_hot_encode(
        str(atom.GetHybridization()), allowable_set, include_unknown_set
    )


def get_atom_total_num_Hs_one_hot(
    atom: RDKitAtom,
    allowable_set: List[int] = DEFAULT_TOTAL_NUM_Hs_SET,
    include_unknown_set: bool = True,
) -> List[float]:
    """Get an one-hot feature of the number of hydrogens which an atom has.

    Parameters
    ----------
    atom: rdkit.Chem.rdchem.Atom
      RDKit atom object
    allowable_set: List[int]
      The number of hydrogens to consider. The default set is `[0, 1, ..., 4]`
    include_unknown_set: bool, default True
      If true, the index of all types not in `allowable_set` is `len(allowable_set)`.

    Returns
    -------
    List[float]
      A one-hot vector of the number of hydrogens which an atom has.
      If `include_unknown_set` is False, the length is `len(allowable_set)`.
      If `include_unknown_set` is True, the length is `len(allowable_set) + 1`.

    """
    return one_hot_encode(atom.GetTotalNumHs(), allowable_set, include_unknown_set)


def get_atom_chirality_one_hot(atom: RDKitAtom) -> List[float]:
    """Get an one-hot feature about an atom chirality type.

    Parameters
    ----------
    atom: rdkit.Chem.rdchem.Atom
      RDKit atom object

    Returns
    -------
    List[float]
      A one-hot vector of the chirality type. The first element
      indicates "R", and the second element indicates "S".

    """
    one_hot = [0.0, 0.0]
    try:
        chiral_type = atom.GetProp("_CIPCode")
        if chiral_type == "R":
            one_hot[0] = 1.0
        elif chiral_type == "S":
            one_hot[1] = 1.0
    except:
        pass
    return one_hot


def get_atom_formal_charge(atom: RDKitAtom) -> List[float]:
    """Get a formal charge of an atom.

    Parameters
    ----------
    atom: rdkit.Chem.rdchem.Atom
      RDKit atom object

    Returns
    -------
    List[float]
      A vector of the formal charge.

    """
    return [float(atom.GetFormalCharge())]


def get_atom_formal_charge_one_hot(
    atom: RDKitAtom,
    allowable_set: List[int] = DEFAULT_FORMAL_CHARGE_SET,
    include_unknown_set: bool = True,
) -> List[float]:
    """Get one hot encoding of formal charge of an atom.

    Parameters
    ----------
    atom: rdkit.Chem.rdchem.Atom
      RDKit atom object
    allowable_set: List[int]
      The degree to consider. The default set is `[-2, -1, ..., 2]`
    include_unknown_set: bool, default True
      If true, the index of all types not in `allowable_set` is `len(allowable_set)`.

    Returns
    -------
    List[float]
      A vector of the formal charge.

    """
    return one_hot_encode(atom.GetFormalCharge(), allowable_set, include_unknown_set)


def get_atom_partial_charge(atom: RDKitAtom) -> List[float]:
    """Get a partial charge of an atom.

    Parameters
    ----------
    atom: rdkit.Chem.rdchem.Atom
      RDKit atom object

    Returns
    -------
    List[float]
      A vector of the parital charge.

    Notes
    -----
    Before using this function, you must calculate `GasteigerCharge`
    like `AllChem.ComputeGasteigerCharges(mol)`.

    """
    gasteiger_charge = atom.GetProp("_GasteigerCharge")
    if gasteiger_charge in ["-nan", "nan", "-inf", "inf"]:
        gasteiger_charge = 0.0
    return [float(gasteiger_charge)]


def get_atom_total_degree_one_hot(
    atom: RDKitAtom,
    allowable_set: List[int] = DEFAULT_TOTAL_DEGREE_SET,
    include_unknown_set: bool = True,
) -> List[float]:
    """Get an one-hot feature of the degree which an atom has.

    Parameters
    ----------
    atom: rdkit.Chem.rdchem.Atom
      RDKit atom object
    allowable_set: List[int]
      The degree to consider. The default set is `[0, 1, ..., 5]`
    include_unknown_set: bool, default True
      If true, the index of all types not in `allowable_set` is `len(allowable_set)`.

    Returns
    -------
    List[float]
      A one-hot vector of the degree which an atom has.
      If `include_unknown_set` is False, the length is `len(allowable_set)`.
      If `include_unknown_set` is True, the length is `len(allowable_set) + 1`.

    """
    return one_hot_encode(atom.GetTotalDegree(), allowable_set, include_unknown_set)


def get_atom_degree_one_hot(
    atom: RDKitAtom,
    allowable_set: List[int] = DEFAULT_TOTAL_DEGREE_SET,
    include_unknown_set: bool = True,
) -> List[float]:
    """Get an one-hot feature of the degree which an atom has.

    Parameters
    ----------
    atom: rdkit.Chem.rdchem.Atom
      RDKit atom object
    allowable_set: List[int]
      The degree to consider. The default set is `[0, 1, ..., 5]`
    include_unknown_set: bool, default True
      If true, the index of all types not in `allowable_set` is `len(allowable_set)`.

    Returns
    -------
    List[float]
      A one-hot vector of the degree which an atom has.
      If `include_unknown_set` is False, the length is `len(allowable_set)`.
      If `include_unknown_set` is True, the length is `len(allowable_set) + 1`.

    """
    return one_hot_encode(atom.GetDegree(), allowable_set, include_unknown_set)


def get_atom_implicit_valence_one_hot(
    atom: RDKitAtom,
    allowable_set: List[int] = DEFAULT_ATOM_IMPLICIT_VALENCE_SET,
    include_unknown_set: bool = True,
) -> List[float]:
    """Get an one-hot feature of implicit valence of an atom.

    Parameters
    ----------
    atom: rdkit.Chem.rdchem.Atom
      RDKit atom object
    allowable_set: List[int]
      Atom implicit valence to consider. The default set is `[0, 1, ..., 6]`
    include_unknown_set: bool, default True
      If true, the index of all types not in `allowable_set` is `len(allowable_set)`.

    Returns
    -------
    List[float]
      A one-hot vector of implicit valence an atom has.
      If `include_unknown_set` is False, the length is `len(allowable_set)`.
      If `include_unknown_set` is True, the length is `len(allowable_set) + 1`.

    """
    return one_hot_encode(atom.GetImplicitValence(), allowable_set, include_unknown_set)


def get_atom_explicit_valence_one_hot(
    atom: RDKitAtom,
    allowable_set: List[int] = DEFAULT_ATOM_EXPLICIT_VALENCE_SET,
    include_unknown_set: bool = True,
) -> List[float]:
    """Get an one-hot feature of explicit valence of an atom.

    Parameters
    ----------
    atom: rdkit.Chem.rdchem.Atom
      RDKit atom object
    allowable_set: List[int]
      Atom explicit valence to consider. The default set is `[1, ..., 6]`
    include_unknown_set: bool, default True
      If true, the index of all types not in `allowable_set` is `len(allowable_set)`.

    Returns
    -------
    List[float]
      A one-hot vector of explicit valence an atom has.
      If `include_unknown_set` is False, the length is `len(allowable_set)`.
      If `include_unknown_set` is True, the length is `len(allowable_set) + 1`.

    """
    return one_hot_encode(atom.GetExplicitValence(), allowable_set, include_unknown_set)


#################################################################
# bond (edge) featurization
#################################################################


def get_bond_type_one_hot(
    bond: RDKitBond,
    allowable_set: List[str] = DEFAULT_BOND_TYPE_SET,
    include_unknown_set: bool = False,
) -> List[float]:
    """Get an one-hot feature of bond type.

    Parameters
    ----------
    bond: rdkit.Chem.rdchem.Bond
      RDKit bond object
    allowable_set: List[str]
      The bond types to consider. The default set is `["SINGLE", "DOUBLE", "TRIPLE", "AROMATIC"]`.
    include_unknown_set: bool, default False
      If true, the index of all types not in `allowable_set` is `len(allowable_set)`.

    Returns
    -------
    List[float]
      A one-hot vector of the bond type.
      If `include_unknown_set` is False, the length is `len(allowable_set)`.
      If `include_unknown_set` is True, the length is `len(allowable_set) + 1`.

    """
    return one_hot_encode(str(bond.GetBondType()), allowable_set, include_unknown_set)


def get_bond_is_in_same_ring_one_hot(bond: RDKitBond) -> List[float]:
    """Get an one-hot feature about whether atoms of a bond is in the same ring or not.

    Parameters
    ----------
    bond: rdkit.Chem.rdchem.Bond
      RDKit bond object

    Returns
    -------
    List[float]
      A one-hot vector of whether a bond is in the same ring or not.

    """
    return [int(bond.IsInRing())]


def get_bond_is_conjugated_one_hot(bond: RDKitBond) -> List[float]:
    """Get an one-hot feature about whether a bond is conjugated or not.

    Parameters
    ----------
    bond: rdkit.Chem.rdchem.Bond
      RDKit bond object

    Returns
    -------
    List[float]
      A one-hot vector of whether a bond is conjugated or not.

    """
    return [int(bond.GetIsConjugated())]


def get_bond_stereo_one_hot(
    bond: RDKitBond,
    allowable_set: List[str] = DEFAULT_BOND_STEREO_SET,
    include_unknown_set: bool = True,
) -> List[float]:
    """Get an one-hot feature of the stereo configuration of a bond.

    Parameters
    ----------
    bond: rdkit.Chem.rdchem.Bond
      RDKit bond object
    allowable_set: List[str]
      The stereo configuration types to consider.
      The default set is `["STEREONONE", "STEREOANY", "STEREOZ", "STEREOE"]`.
    include_unknown_set: bool, default True
      If true, the index of all types not in `allowable_set` is `len(allowable_set)`.

    Returns
    -------
    List[float]
      A one-hot vector of the stereo configuration of a bond.
      If `include_unknown_set` is False, the length is `len(allowable_set)`.
      If `include_unknown_set` is True, the length is `len(allowable_set) + 1`.

    """
    return one_hot_encode(str(bond.GetStereo()), allowable_set, include_unknown_set)


def get_bond_graph_distance_one_hot(
    bond: RDKitBond,
    graph_dist_matrix: np.ndarray,
    allowable_set: List[int] = DEFAULT_GRAPH_DISTANCE_SET,
    include_unknown_set: bool = True,
) -> List[float]:
    """Get an one-hot feature of graph distance.

    Parameters
    ----------
    bond: rdkit.Chem.rdchem.Bond
      RDKit bond object
    graph_dist_matrix: np.ndarray
      The return value of `Chem.GetDistanceMatrix(mol)`. The shape is `(num_atoms, num_atoms)`.
    allowable_set: List[int]
      The graph distance types to consider. The default set is `[1, 2, ..., 7]`.
    include_unknown_set: bool, default False
      If true, the index of all types not in `allowable_set` is `len(allowable_set)`.

    Returns
    -------
    List[float]
      A one-hot vector of the graph distance.
      If `include_unknown_set` is False, the length is `len(allowable_set)`.
      If `include_unknown_set` is True, the length is `len(allowable_set) + 1`.

    """
    graph_dist = graph_dist_matrix[bond.GetBeginAtomIdx(), bond.GetEndAtomIdx()]
    return one_hot_encode(graph_dist, allowable_set, include_unknown_set)



import os

"""
Configuration variables for Jaqpotpy
"""

"""
x -> Test variable
verbose -> Verbose outputs. eg descriptor creation, model training
global_seed -> Seed for various descriptors and models
version -> Jaqpotpy version
"""

x = 0
verbose = True
global_seed = 42
# version = jaqpotpy.__version__



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.models.feature_possible_value import FeaturePossibleValue

class TestFeaturePossibleValue(unittest.TestCase):
    """FeaturePossibleValue unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> FeaturePossibleValue:
        """Test FeaturePossibleValue
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `FeaturePossibleValue`
        """
        model = FeaturePossibleValue()
        if include_optional:
            return FeaturePossibleValue(
                key = 'value',
                value = 'value'
            )
        else:
            return FeaturePossibleValue(
                key = 'value',
                value = 'value',
        )
        """

    def testFeaturePossibleValue(self):
        """Test FeaturePossibleValue"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



from jaqpotpy.models.trainers.base_trainer import TorchModelTrainer
from jaqpotpy.models.trainers.binary_trainer import BinaryGraphModelTrainer
from jaqpotpy.models.trainers.regression_trainer import RegressionGraphModelTrainer



"""Author: Ioannis Pitoskas (jpitoskas@gmail.com)"""


def get_installed_packages() -> dict:
    """Retrieves a dictionary of installed packages and their versions using pip.freeze.

    Returns
    -------
        dict: A dictionary with package names as keys and versions as values.

    """
    try:
        from pip._internal.operations import freeze
    except ImportError:  # pip < 10.0
        from pip.operations import freeze

    pip_freeze_rows = freeze.freeze()
    packages_dict = {}
    for pkg in pip_freeze_rows:
        name, version = pkg.split("==")
        packages_dict[name] = version

    return packages_dict



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

from typing import Any, Optional
from typing_extensions import Self

class OpenApiException(Exception):
    """The base exception class for all OpenAPIExceptions"""


class ApiTypeError(OpenApiException, TypeError):
    def __init__(self, msg, path_to_item=None, valid_classes=None,
                 key_type=None) -> None:
        """ Raises an exception for TypeErrors

        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (list): a list of keys an indices to get to the
                                 current_item
                                 None if unset
            valid_classes (tuple): the primitive classes that current item
                                   should be an instance of
                                   None if unset
            key_type (bool): False if our value is a value in a dict
                             True if it is a key in a dict
                             False if our item is an item in a list
                             None if unset
        """
        self.path_to_item = path_to_item
        self.valid_classes = valid_classes
        self.key_type = key_type
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiTypeError, self).__init__(full_msg)


class ApiValueError(OpenApiException, ValueError):
    def __init__(self, msg, path_to_item=None) -> None:
        """
        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (list) the path to the exception in the
                received_data dict. None if unset
        """

        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiValueError, self).__init__(full_msg)


class ApiAttributeError(OpenApiException, AttributeError):
    def __init__(self, msg, path_to_item=None) -> None:
        """
        Raised when an attribute reference or assignment fails.

        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (None/list) the path to the exception in the
                received_data dict
        """
        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiAttributeError, self).__init__(full_msg)


class ApiKeyError(OpenApiException, KeyError):
    def __init__(self, msg, path_to_item=None) -> None:
        """
        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (None/list) the path to the exception in the
                received_data dict
        """
        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiKeyError, self).__init__(full_msg)


class ApiException(OpenApiException):

    def __init__(
        self, 
        status=None, 
        reason=None, 
        http_resp=None,
        *,
        body: Optional[str] = None,
        data: Optional[Any] = None,
    ) -> None:
        self.status = status
        self.reason = reason
        self.body = body
        self.data = data
        self.headers = None

        if http_resp:
            if self.status is None:
                self.status = http_resp.status
            if self.reason is None:
                self.reason = http_resp.reason
            if self.body is None:
                try:
                    self.body = http_resp.data.decode('utf-8')
                except Exception:
                    pass
            self.headers = http_resp.getheaders()

    @classmethod
    def from_response(
        cls, 
        *, 
        http_resp, 
        body: Optional[str], 
        data: Optional[Any],
    ) -> Self:
        if http_resp.status == 400:
            raise BadRequestException(http_resp=http_resp, body=body, data=data)

        if http_resp.status == 401:
            raise UnauthorizedException(http_resp=http_resp, body=body, data=data)

        if http_resp.status == 403:
            raise ForbiddenException(http_resp=http_resp, body=body, data=data)

        if http_resp.status == 404:
            raise NotFoundException(http_resp=http_resp, body=body, data=data)

        if 500 <= http_resp.status <= 599:
            raise ServiceException(http_resp=http_resp, body=body, data=data)
        raise ApiException(http_resp=http_resp, body=body, data=data)

    def __str__(self):
        """Custom error messages for exception"""
        error_message = "({0})\n"\
                        "Reason: {1}\n".format(self.status, self.reason)
        if self.headers:
            error_message += "HTTP response headers: {0}\n".format(
                self.headers)

        if self.data or self.body:
            error_message += "HTTP response body: {0}\n".format(self.data or self.body)

        return error_message


class BadRequestException(ApiException):
    pass


class NotFoundException(ApiException):
    pass


class UnauthorizedException(ApiException):
    pass


class ForbiddenException(ApiException):
    pass


class ServiceException(ApiException):
    pass


def render_path(path_to_item):
    """Returns a string representation of a path"""
    result = ""
    for pth in path_to_item:
        if isinstance(pth, int):
            result += "[{0}]".format(pth)
        else:
            result += "['{0}']".format(pth)
    return result



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.models.get_models200_response import GetModels200Response

class TestGetModels200Response(unittest.TestCase):
    """GetModels200Response unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> GetModels200Response:
        """Test GetModels200Response
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `GetModels200Response`
        """
        model = GetModels200Response()
        if include_optional:
            return GetModels200Response(
                content = [
                    jaqpotpy.api.openapi.models.model_summary.ModelSummary(
                        id = 0, 
                        name = 'My Model', 
                        visibility = 'PUBLIC', 
                        description = 'A description of your model', 
                        creator = jaqpotpy.api.openapi.models.user.User(
                            id = '', 
                            username = '', 
                            first_name = '', 
                            last_name = '', 
                            email = '', 
                            email_verified = True, ), 
                        type = 'SKLEARN', 
                        dependent_features_length = 56, 
                        independent_features_length = 56, 
                        shared_with_organizations = [
                            jaqpotpy.api.openapi.models.organization_summary.OrganizationSummary(
                                id = 0, 
                                name = 'My Organization', )
                            ], 
                        created_at = '2023-01-01T12:00Z', 
                        updated_at = '2023-01-01T12:00:00Z', )
                    ],
                total_elements = 56,
                total_pages = 56,
                page_size = 56,
                page_number = 56
            )
        else:
            return GetModels200Response(
        )
        """

    def testGetModels200Response(self):
        """Test GetModels200Response"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



import numpy as np
from typing import Union, Tuple
import os
import tempfile
import tarfile
import zipfile
from urllib.request import urlretrieve
from typing import Optional


def pad_array(
    x: np.ndarray, shape: Union[Tuple, int], fill: float = 0.0, both: bool = False
) -> np.ndarray:
    """Pad an array with a fill value.

    Parameters
    ----------
    x: np.ndarray
      A numpy array.
    shape: Tuple or int
      Desired shape. If int, all dimensions are padded to that size.
    fill: float, optional (default 0.0)
      The padded value.
    both: bool, optional (default False)
      If True, split the padding on both sides of each axis. If False,
      padding is applied to the end of each axis.

    Returns
    -------
    np.ndarray
      A padded numpy array

    """
    x = np.asarray(x)
    if not isinstance(shape, tuple):
        shape = tuple(shape for _ in range(x.ndim))
    pad = []
    for i in range(x.ndim):
        diff = shape[i] - x.shape[i]
        assert diff >= 0
        if both:
            a, b = divmod(diff, 2)
            b += a
            pad.append((a, b))
        else:
            pad.append((0, diff))
    pad = tuple(pad)  # type: ignore
    x = np.pad(x, pad, mode="constant", constant_values=fill)
    return x


def get_data_dir() -> str:
    """Get the DeepChem data directory.

    Returns
    -------
    str
      The default path to store DeepChem data. If you want to
      change this path, please set your own path to `DEEPCHEM_DATA_DIR`
      as an environment variable.

    """
    if "DEEPCHEM_DATA_DIR" in os.environ:
        return os.environ["DEEPCHEM_DATA_DIR"]
    return tempfile.gettempdir()


def download_url(url: str, dest_dir: str = get_data_dir(), name: Optional[str] = None):
    """Download a file to disk.

    Parameters
    ----------
    url: str
      The URL to download from
    dest_dir: str
      The directory to save the file in
    name: str
      The file name to save it as.  If omitted, it will try to extract a file name from the URL

    """
    if name is None:
        name = url
        if "?" in name:
            name = name[: name.find("?")]
        if "/" in name:
            name = name[name.rfind("/") + 1 :]
    if not os.path.exists(dest_dir):
        os.makedirs(dest_dir)
    urlretrieve(url, os.path.join(dest_dir, name))


def untargz_file(file: str, dest_dir: str = get_data_dir(), name: Optional[str] = None):
    """Untar and unzip a .tar.gz file to disk.

    Parameters
    ----------
    file: str
      The filepath to decompress
    dest_dir: str
      The directory to save the file in
    name: str
      The file name to save it as.  If omitted, it will use the file name

    """
    if name is None:
        name = file
    tar = tarfile.open(name)
    tar.extractall(path=dest_dir)
    tar.close()


def unzip_file(file: str, dest_dir: str = get_data_dir(), name: Optional[str] = None):
    """Unzip a .zip file to disk.

    Parameters
    ----------
    file: str
      The filepath to decompress
    dest_dir: str
      The directory to save the file in
    name: str
      The directory name to unzip it to.  If omitted, it will use the file name

    """
    if name is None:
        name = file
    if dest_dir is None:
        dest_dir = os.path.join(get_data_dir, name)
    with zipfile.ZipFile(file, "r") as zip_ref:
        zip_ref.extractall(dest_dir)



"""Pytorch Utilities.
This file contains utilities that compute Datastractures adn other for Pytorch.
"""

import numpy as np


def to_torch_graph_data_and_y(g, y):
    """Returns torch graph data from jaqpot graph_data and adds y"""
    import torch
    from torch_geometric.data import Data

    torch_graph = Data(
        x=torch.FloatTensor(g.node_features),
        edge_index=torch.LongTensor(g.edge_index),
        edge_attr=g.edge_features,
        num_nodes=g.num_nodes,
        y=np.array([y]),
    )
    return torch_graph


def to_torch_graph_data(g):
    """Returns torch graph data from jaqpot graph_data without endpoint"""
    import torch
    from torch_geometric.data import Data

    torch_graph = Data(
        x=torch.FloatTensor(g.node_features),
        edge_index=torch.LongTensor(g.edge_index),
        edge_attr=g.edge_features,
        num_nodes=g.num_nodes,
    )
    return torch_graph


def to_torch_graph_data_array_and_regr_y(mol_g, y):
    """Returns torch graph data from array of jaqpot graph_data and y"""
    import torch
    from torch_geometric.data import Data

    datas = []
    i = 0
    for g in mol_g:
        dato = Data(
            x=torch.FloatTensor(g.node_features),
            edge_index=torch.LongTensor(g.edge_index),
            edge_attr=g.edge_features,
            num_nodes=g.num_nodes,
            y=torch.Tensor([y[i]]),
        )
        datas.append(dato)
        i += 1
    return datas


def to_torch_graph_data_array_and_class_y(mol_g, y):
    """Returns torch graph data from array of jaqpot graph_data and y"""
    import torch
    from torch_geometric.data import Data

    datas = []
    i = 0
    for g in mol_g:
        dato = Data(
            x=torch.FloatTensor(g.node_features),
            edge_index=torch.LongTensor(g.edge_index),
            edge_attr=g.edge_features,
            num_nodes=g.num_nodes,
            y=torch.LongTensor([y[i]]),
        )
        datas.append(dato)
        i += 1
    return datas


def to_torch_graph_data_array(mol_g):
    """Returns torch graph data from array of jaqpot graph_data"""
    import torch
    from torch_geometric.data import Data

    datas = []
    for g in mol_g:
        dato = Data(
            x=torch.FloatTensor(g.node_features),
            edge_index=torch.LongTensor(g.edge_index),
            edge_attr=g.edge_features,
            num_nodes=g.num_nodes,
        )
        datas.append(dato)
    return datas



# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.models.model_extra_config import ModelExtraConfig

class TestModelExtraConfig(unittest.TestCase):
    """ModelExtraConfig unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ModelExtraConfig:
        """Test ModelExtraConfig
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `ModelExtraConfig`
        """
        model = ModelExtraConfig()
        if include_optional:
            return ModelExtraConfig(
                torch_config = {
                    'key' : None
                    },
                preprocessors = [
                    jaqpotpy.api.openapi.models.transformer.Transformer(
                        name = 'StandardScaler', 
                        config = {
                            'key' : None
                            }, )
                    ],
                featurizers = [
                    jaqpotpy.api.openapi.models.transformer.Transformer(
                        name = 'StandardScaler', 
                        config = {
                            'key' : None
                            }, )
                    ],
                doa = [
                    jaqpotpy.api.openapi.models.transformer.Transformer(
                        name = 'StandardScaler', 
                        config = {
                            'key' : None
                            }, )
                    ]
            )
        else:
            return ModelExtraConfig(
        )
        """

    def testModelExtraConfig(self):
        """Test ModelExtraConfig"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
